/*****************************************************************
 * Copyright (c) ParaSoft Corporation 1991.  All rights reserved.*
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft       *
 * The copyright notice above does not evidence any              *
 * actual or intended publication of such source code.           *
 *****************************************************************/

#include "extern.h"

typedef struct Line {
	int c;
	char *line;
	int curlen;
	int linelen;
} Line;
#define LN (Line*) 0

#define MCTMP  100
#define MCL		72
#define DIG0	48

#define BUF_LEN	200

/**/ U_DEFINE_STACK (lines, Line*, 10)

static flag_blanks_in_output = 1;
static Line *line;

static Line *line_malloc ()
{
	Line *line;

	line = (Line*) U_malloc (sizeof (Line));
	line->line = (char*) U_malloc (sizeof (char) * (BUF_LEN + 1));
	line->curlen = BUF_LEN;

	return line;
}

ostart ()
{
	line = line_malloc ();
	U_stack_push_lines (line);
}

oend ()
{
	int i, nc;

	for (nc = 0, i = 0; i < line->c; i++, nc++) {
		if (line->line [i] == '\007') {
			fputc ('\n', ofd);
			nc = -1;
		}
		else {
			if (nc == MCL) {
				fprintf (ofd, "\n     &");
				nc = 6;
			}
	
			fputc (line->line [i], ofd);
		}
	}

	if (lines_top > 0) {
		U_stack_pop_lines ();
		line = lines [lines_top - 1];
	}
	else {
		sprintf (serr, "lines_top = %d", lines_top);
		syserr (serr);
	}
}

odon ()
{
	och ('\007');
	line->linelen = 0;
}

oflush ()
{
}

eof ()
{
	if (line->c)
		odon ();
}

och (c)
char c;
{
	if (line->c > line->curlen - 1) {
		char *nline;

		nline = (char*) U_malloc (sizeof (char) * (line->curlen + BUF_LEN + 1));
		strcpy (nline, line->line);
		U_free (line->line);
		line->line = nline;
		line->curlen = line->curlen + BUF_LEN;
	}

	line->line [line->c++] = c;
	line->linelen++;
}

ostr(str)  /* outputs string */
char str[];
{
    int i;
	char prev;

	if (flag_blanks_in_output)
		if (*str != ')' && *str != ',' && line->c > 0
		&& (prev = line->line[line->c-1]) != ' ' && prev != '(' && prev != '\t')
			och (' ');

    if (str)
        for (i = 0; str[i] != '\0'; i++)
             och (str[i]);
}

ostrnb(str)  /* outputs string */
char str[];
{
    int i;
	char prev;

    if (str)
        for (i = 0; str[i] != '\0'; i++)
             och (str[i]);
}

onam (str)
char *str;
{
	ostr (name_out (str));
}

olab (lab)
Node *lab;
{
	char *str = lab->com.str;

	if (*str == '#')
		ostr (name_out (str));
	else
		ostr (str);
}

oint(n)  /* outputs integer */
long n;
{
    char chars[MCTMP];
    int i;
	int blank = 1;
	long m;

    m = abs(n);
    i = -1;
    do {
         i++;
         chars[i] = (m % 10) + DIG0;
         m = m / 10;
    } 
    while (m > 0 && i < MCTMP);

    if (n < 0) {
		if (flag_blanks_in_output) {
			ochb ('-');
			blank = 0;
		}
		else
			och('-');
	}
    for ( ; i >= 0; i--) {
		if (flag_blanks_in_output && blank) {
			ochb (chars[i]);
			blank = 0;
		}
		else
			och (chars[i]);
	}
}

ochb (c)  /* output character taking care of blank spacing */
char c;
{
	char prev;

	if (flag_blanks_in_output)
		if (c != ')' && c != ',' && line->c > 0
		&& (prev = line->line[line->c-1]) != ' ' && prev != '(' && prev != '\t')
			och (' ');

	och (c);
}

otab()  /* outputs a tab */
{
	while (line->linelen < 6)
		och (' ');
}

output_check ()
{
#ifdef CHECK

	if (lines_top)
		SYSERR;

#endif
}

output_init ()
{
}

output_delete_output_file ()
{
	if (output_filename != CN)
		unlink (output_filename);
}

ozero ()
{
	ochb ('0');
}

oone ()
{
	ochb ('1');
}

o_continue (label)
Node *label;
{
	ostart ();
	olab (label);
	otab ();
	oCONTINUE ();
	oDon ();
}

o_goto (label)
Node *label;
{
	oTab ();
	oGO_TO ();
	olab (label);
	oDon ();
}

oendif ()
{
	ostart ();
	otab ();
	oEND_IF ();
	odon ();
	oend ();
}
	
/*************************************************************************/

oTHEN ()
{
	ostr ("THEN");
}

/* ZEKI */

oEND_FORALL ()
{
	ostr ("ENDFOR");
}

oTab ()
{
	ostart ();
	otab ();
}

oDon ()
{
	odon ();
	oend ();
}
